@@ -1,7 +1,7 @@
 package Bencher::Backend;
 
-our $DATE = '2016-03-15'; # DATE
-our $VERSION = '0.39'; # VERSION
+our $DATE = '2016-03-16'; # DATE
+our $VERSION = '0.40'; # VERSION
 
 use 5.010001;
 use strict;
@@ -9,7 +9,6 @@ use warnings;
 use Log::Any::IfLOG '$log';
 
 use Data::Dmp;
-use List::MoreUtils qw(firstidx minmax uniq);
 
 our %SPEC;
 
@@ -346,8 +345,6 @@ sub _gen_items {
     $parsed->{items} = [];
     my @permute;
 
-    # XXX allow permutation of module path
-
     my $participants;
     my $datasets;
     my $module_startup = $pargs->{module_startup} // $parsed->{module_startup};
@@ -355,7 +352,6 @@ sub _gen_items {
     my @modules = _get_participant_modules($parsed);
 
     if ($module_startup) {
-
         return [412, "There are no modules to benchmark ".
                     "the startup overhead of"]
             unless @modules;
@@ -376,6 +372,7 @@ sub _gen_items {
                     seq  => $i,
                     name => $mod,
                     type => 'command',
+                    module => $mod,
                     perl_cmdline => ["-M$mod", "-e1"],
                 };
             }
@@ -393,8 +390,10 @@ sub _gen_items {
         my @perls;
         if ($pargs->{multiperl}) {
             require App::perlbrew;
-            my $pb = App::perlbrew->new;
-            @perls = $pb->installed_perls;
+            @perls = grep {$_->{has_bencher}} _list_perls();
+            return [412, "Can't multiperl because no perl has Bencher installed"]
+                unless @perls;
+
             if ($pargs->{include_perls} && @{ $pargs->{include_perls} }) {
                 @perls = grep {
                     my $p = $_;
@@ -477,8 +476,9 @@ sub _gen_items {
     my %ds_arg_values; # key=ds seq, val=hash(key=arg name, val=arg values)
 
     my $iter = Permute::Named::Iter::permute_named_iter(@permute);
-    my $item_seq = -1;
+    my $item_seq = 0;
     my $items = [];
+    my %item_mems; # key=item key, value=1
   ITER:
     while (my $h = $iter->()) {
         $log->tracef("iter returns: %s", $h);
@@ -581,10 +581,15 @@ sub _gen_items {
             };
         }
 
+        # don't permute module versions if participant doesn't involve said
+        # module
+        if ($pargs->{multimodver} && defined($h->{modver}) &&
+                $pargs->{multimodver} ne ($p->{module} // '')) {
+            $h->{modver} = '';
+        }
+
       ITER_ARGS:
         while (my $h_args = $iter_args->()) {
-            $item_seq++;
-
             my $args;
             if ($ds && $ds->{args}) {
                 $args = { %{$ds->{args}} };
@@ -614,7 +619,7 @@ sub _gen_items {
                 } elsif (defined $p->{perl_cmdline}) {
                     if (ref($p->{perl_cmdline}) eq 'ARRAY') {
                         @cmd = ($perl_exes{$h->{perl}},
-                                ($h->{modver} ? @{$perl_opts{$h->{modver}}} : ()),
+                                ($h->{modver} ? @{$perl_opts{$h->{modver}} // []} : ()),
                                 @{ $p->{perl_cmdline} });
                         $shell = 0;
                     } else {
@@ -622,7 +627,7 @@ sub _gen_items {
                             join(
                                 " ",
                                 $perl_exes{$h->{perl}},
-                                ($h->{modver} ? map {String::ShellQuote::shell_quote($_)} @{$perl_opts{$h->{modver}}} : ()),
+                                ($h->{modver} ? map {String::ShellQuote::shell_quote($_)} @{$perl_opts{$h->{modver}} // []} : ()),
                                 $p->{perl_cmdline},
                             )
                         );
@@ -660,7 +665,7 @@ sub _gen_items {
                     if (ref($p->{perl_cmdline_template}) eq 'ARRAY') {
                         @cmd = (
                             $perl_exes{$h->{perl}},
-                            ($h->{modver} ? @{$perl_opts{$h->{modver}}} : ()),
+                            ($h->{modver} ? @{$perl_opts{$h->{modver}} // []} : ()),
                             map {
                                 my $el = $_;
                                 $el =~ s/\<(\w+(?::\w+)?)\>/$template_vars->{$1}/g;
@@ -672,7 +677,7 @@ sub _gen_items {
                         my $cmd = join(
                             " ",
                             $perl_exes{$h->{perl}},
-                            ($h->{modver} ? map {String::ShellQuote::shell_quote($_)} @{$perl_opts{$h->{modver}}} : ()),
+                            ($h->{modver} ? map {String::ShellQuote::shell_quote($_)} @{$perl_opts{$h->{modver}} // []} : ()),
                             $p->{perl_cmdline_template},
                         );
                         $cmd =~ s/\<(\w+(?::\w+)?)\>/String::ShellQuote::shell_quote($template_vars->{$1})/eg;
@@ -734,7 +739,6 @@ sub _gen_items {
             }
 
             my $item = {
-                seq  => $item_seq,
                 _code => $code,
                 _permute => $h,
                 ((_permute_args => $h_args) x !!$ds->{args}),
@@ -760,6 +764,18 @@ sub _gen_items {
                 }
             }
 
+            # skip duplicate items
+            my $key = dmp [map { $item->{$_} }
+                               grep { !/^_/ }
+                               sort keys %$item];
+            $log->tracef("item key=%s", $key);
+            if ($item_mems{$key}++) {
+                $log->tracef("Duplicate key, skipped item, recycling seq number %d", $item_seq);
+                next ITER;
+            }
+
+            $item->{seq} = $item_seq++;
+
             push @$items, $item;
 
             last ITER_ARGS unless $ds->{args};
@@ -1015,20 +1031,48 @@ sub _complete_item {
     );
 }
 
-sub _complete_perl {
-    require Complete::Util;
-
-    my %args = @_;
-    my $word    = $args{word} // '';
+# list installed perls, and check each perl if Bencher::Backend is installed
+sub _list_perls {
+    require Capture::Tiny;
 
     eval { require App::perlbrew; 1 };
     return undef if $@;
 
     my $pb = App::perlbrew->new;
     my @perls = $pb->installed_perls;
+    for my $perl (@perls) {
+        my @cmd = (
+            $perl->{executable},
+            "-MBencher::Backend",
+            "-e'print \$Bencher::Backend::VERSION'",
+        );
+        my ($stdout, $stderr, @res) =
+            Capture::Tiny::capture(sub { system @cmd });
+        if ($stderr || $?) {
+            $perl->{has_bencher} = 0;
+            $perl->{bencher_version} = undef;
+        } else {
+            $perl->{has_bencher} = 1;
+            $perl->{bencher_version} = $stdout;
+        }
+    }
+
+    @perls;
+}
+
+sub _complete_perl {
+    require Complete::Util;
+
+    my %args = @_;
+    my $word    = $args{word} // '';
+
+    my @perls = _list_perls();
+
     local $Complete::Common::OPT_FUZZY = 0;
     Complete::Util::complete_array_elem(
-        word => $word, array => [map {$_->{name}} @perls]);
+        word => $word,
+        array => [map {$_->{name}} grep {$_->{has_bencher}} @perls],
+    );
 }
 
 my $_alias_spec_add_dataset = {
@@ -1214,6 +1258,7 @@ _
         action => {
             schema => ['str*', {
                 in=>[qw/
+                           list-perls
                            list-scenario-modules
                            show-scenario
                            list-participants
@@ -1229,6 +1274,11 @@ _
             default => 'bench',
             cmdline_aliases => {
                 a => {},
+                list_perls => {
+                    is_flag => 1,
+                    summary => 'Shortcut for -a list-perls',
+                    code => sub { $_[0]{action} = 'list-perls' },
+                },
                 list_scenario_modules => {
                     is_flag => 1,
                     summary => 'Shortcut for -a list-scenario-modules',
@@ -1508,11 +1558,13 @@ _
             description => <<'_',
 
 Requires `App::perlbrew` to be installed. Will use installed perls from the
-perlbrew installation. Use `--include-perl` and `--exclude-perl` to include and
-exclude which perls you want.
+perlbrew installation. Each installed perl must have `Bencher::Backend` module
+installed (in addition to having all modules that you want to benchmark,
+obviously).
 
-Each installed perl must have `Bencher::Backend` module installed (in addition
-to having all modules that you want to benchmark, obviously).
+By default, only perls having Bencher::Backend will be included. Use
+`--include-perl` and `--exclude-perl` to include and exclude which perls you
+want.
 
 Also note that due to the way this is currently implemented, benchmark code that
 contains closures (references to variables outside the code) won't work.
@@ -1627,6 +1679,33 @@ sub bencher {
     my $action = $args{action};
     my $envres;
 
+    if ($action eq 'list-perls') {
+        my @perls = _list_perls();
+        my @res;
+        for my $perl (@perls) {
+            if ($args{detail}) {
+                push @res, {
+                    name            => $perl->{name},
+                    version         => $perl->{version},
+                    has_bencher     => $perl->{has_bencher},
+                    bencher_version => $perl->{bencher_version},
+                };
+            } else {
+                push @res, $perl->{name};
+            }
+        }
+        my %resmeta;
+        $resmeta{'table.fields'} = [
+            'name',
+            'version',
+            'has_bencher',
+            'bencher_version',
+        ] if $args{detail};
+        $envres =
+            [200, "OK", \@res, \%resmeta];
+        goto L_END;
+    }
+
     if ($action eq 'list-scenario-modules') {
         require PERLANCAR::Module::List;
         my $mods = PERLANCAR::Module::List::list_modules(
@@ -2039,13 +2118,19 @@ sub bencher {
 
             my $tempdir = File::Temp::tempdir(CLEANUP => $log->is_debug ? 0:1);
 
+            my %item_mems; # key = item seq
             for my $perl (sort keys %perl_exes) {
                 for my $modver (sort keys %perl_opts) {
                     my $scd_path = "$tempdir/scenario-$perl";
-                    $sc->{items} = [grep {
-                        $_->{perl} eq $perl &&
-                            (length($modver) ? $_->{modver} eq $modver : 1)
-                        } @$items];
+                    $sc->{items} = [];
+                    for my $it (@$items) {
+                        next unless $it->{perl} eq $perl;
+                        next unless !length($it->{modver}) ||
+                            $it->{modver} eq $modver;
+                        next if $item_mems{$it->{seq}}++; # avoid duplicate item
+                        push @{$sc->{items}}, $it;
+                    }
+                    #use DD; dd {perl=>$perl, modver=>$modver, items=>$sc->{items}};
                     $log->debugf("Creating scenario dump file for %s (modver %s) at %s", $perl, $modver, $scd_path);
                     open my($fh), ">", $scd_path or die "Can't open file $scd_path: $!";
                     print $fh dmp($sc), ";\n";
@@ -2056,7 +2141,7 @@ sub bencher {
                         $perl_exes{$perl},
                         "-MBencher::Backend",
                         "-MData::Dmp",
-                        @{ $perl_opts{$modver} },
+                        @{ $perl_opts{$modver} // [] },
                         "-e'print dmp(Bencher::Backend::bencher(action=>q[bench], precision=>$precision, scenario_file=>q[$scd_path], return_meta=>0))' > '$res_path'",
                     );
                     $log->debugf("Running %s ...", $cmd);
@@ -2204,7 +2289,7 @@ Bencher::Backend - Backend for Bencher
 
 =head1 VERSION
 
-This document describes version 0.39 of Bencher::Backend (from Perl distribution Bencher-Backend), released on 2016-03-15.
+This document describes version 0.40 of Bencher::Backend (from Perl distribution Bencher-Backend), released on 2016-03-16.
 
 =head1 FUNCTIONS
 
@@ -2363,11 +2448,13 @@ Currently only one module can be multi version.
 Benchmark against multiple perls.
 
 Requires C<App::perlbrew> to be installed. Will use installed perls from the
-perlbrew installation. Use C<--include-perl> and C<--exclude-perl> to include and
-exclude which perls you want.
+perlbrew installation. Each installed perl must have C<Bencher::Backend> module
+installed (in addition to having all modules that you want to benchmark,
+obviously).
 
-Each installed perl must have C<Bencher::Backend> module installed (in addition
-to having all modules that you want to benchmark, obviously).
+By default, only perls having Bencher::Backend will be included. Use
+C<--include-perl> and C<--exclude-perl> to include and exclude which perls you
+want.
 
 Also note that due to the way this is currently implemented, benchmark code that
 contains closures (references to variables outside the code) won't work.
